@@ -74,6 +74,7 @@ extern "C" {
 #endif
 
 /* Declared in int2str() */
+extern const char _dig_vec_base62[];
 extern const char _dig_vec_upper[];
 extern const char _dig_vec_lower[];
 
@@ -201,7 +202,7 @@ extern ulonglong strtoull(const char *str, char **ptr, int base);
 
 #ifdef __cplusplus
 #include <type_traits>
-template<typename T> inline const char *_swl_check(T s)
+template<typename T> inline constexpr const char *_swl_check(T s)
 {
   static_assert(std::is_same<T, const char (&)[sizeof(T)]>::value
              || std::is_same<T, const char [sizeof(T)]>::value,
@@ -219,6 +220,15 @@ template<typename T> inline const char *_swl_check(T s)
 
 typedef struct st_mysql_const_lex_string LEX_CSTRING;
 
+#ifdef  __cplusplus
+static inline constexpr
+LEX_CSTRING operator"" _LEX_CSTRING(const char *str, size_t length)
+{
+  return LEX_CSTRING{str, length};
+}
+#endif /* __cplusplus */
+
+
 /* A variant with const and unsigned */
 struct st_mysql_const_unsigned_lex_string
 {
@@ -239,42 +249,65 @@ static inline void lex_string_set3(LEX_CSTRING *lex_str, const char *c_str,
   lex_str->length= len;
 }
 
-/*
-  Copies src into dst and ensures dst is a NULL terminated C string.
+/**
+  Copies a string.
 
-  Returns 1 if the src string was truncated due to too small size of dst.
-  Returns 0 if src completely fit within dst. Pads the remaining dst with '\0'
-
-  Note: dst_size must be > 0
+  @param dst        destination buffer, will be NUL padded.
+  @param dst_size   size of dst buffer, must be > 0
+  @param src        NUL terminated source string
 */
-static inline int safe_strcpy(char *dst, size_t dst_size, const char *src)
+static inline void safe_strcpy(char *dst, size_t dst_size, const char *src)
 {
-  memset(dst, '\0', dst_size);
-  strncpy(dst, src, dst_size - 1);
-  /*
-     If the first condition is true, we are guaranteed to have src length
-     >= (dst_size - 1), hence safe to access src[dst_size - 1].
-  */
-  if (dst[dst_size - 2] != '\0' && src[dst_size - 1] != '\0')
-    return 1; /* Truncation of src. */
+  DBUG_ASSERT(dst_size > 0);
+
+  /* 1) IF there is a 0 byte in the first dst_size bytes of src, strncpy will
+   *    0-terminate dst, and pad dst with additional 0 bytes out to dst_size.
+   *
+   * 2) IF there is no 0 byte in the first dst_size bytes of src, strncpy will
+   *    copy dst_size bytes, and the final byte won't be 0.
+   */
+
+  strncpy(dst, src, dst_size);
+  dst[dst_size - 1]= 0;
+}
+
+/**
+  Copies a string, checking for truncation.
+
+  @param dst        destination buffer, will be NUL padded.
+  @param dst_size   size of dst buffer, must be > 0
+  @param src        NUL terminated source string
+
+  @retval 1 if the src string was truncated due to too small size of dst.
+  @retval 0 if src completely fit within dst,
+*/
+static inline int safe_strcpy_truncated(char *dst, size_t dst_size,
+                                        const char *src)
+{
+  DBUG_ASSERT(dst_size > 0);
+
+  strncpy(dst, src, dst_size);
+  if (dst[dst_size - 1])
+  {
+    dst[dst_size - 1]= 0;
+    return 1;
+  }
   return 0;
 }
 
-/*
-  Appends src to dst and ensures dst is a NULL terminated C string.
+/**
+  Appends src to dst and ensures dst is a NUL terminated C string.
 
-  Returns 1 if the src string was truncated due to too small size of dst.
-  Returns 0 if src completely fit within the remaining dst space. Pads the
-            remaining dst with '\0'.
-
-  Note: dst_size must be > 0
+  @retval 1 if the src string was truncated due to too small size of dst.
+  @retval 0 if src completely fit within the remaining dst space,
+  including NUL termination.
 */
 static inline int safe_strcat(char *dst, size_t dst_size, const char *src)
 {
   size_t init_len= strlen(dst);
-  if (init_len >= dst_size - 1)
+  if (init_len > dst_size)
     return 1;
-  return safe_strcpy(dst + init_len, dst_size - init_len, src);
+  return safe_strcpy_truncated(dst + init_len, dst_size - init_len, src);
 }
 
 #ifdef __cplusplus
